<?php

namespace Drupal\entity_usage_explorer;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Entity\EntityInterface;
use Twig\Extension\AbstractExtension;
use Twig\TwigFunction;

/**
 * Provides custom Twig functions for the Entity Usage Explorer module.
 */
class UsageHelper extends AbstractExtension {

  /**
   * The entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected EntityTypeManagerInterface $entityTypeManager;

  /**
   * Constructs a UsageHelper object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager service.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager) {
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public function getFunctions(): array {
    return [
      new TwigFunction('loadParagraphParent', [$this, 'loadParagraphParent']),
    ];
  }

  /**
   * Loads the parent entity of a paragraph.
   *
   * @param int $pid
   *   The paragraph ID.
   *
   * @return \Drupal\Core\Entity\EntityInterface|null
   *   The parent entity if found, NULL otherwise.
   */
  public function loadParagraphParent(int $pid): ?EntityInterface {
    $paragraph = $this->entityTypeManager->getStorage('paragraph')->load($pid);

    if ($paragraph instanceof EntityInterface && method_exists($paragraph, 'getParentEntity')) {
      return $paragraph?->getParentEntity();
    }

    return NULL;
  }

}
